package net.knarcraft.placeholdersigns.command;

import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.util.TabCompletionHelper;
import net.knarcraft.placeholdersigns.PlaceholderSigns;
import net.knarcraft.placeholdersigns.config.PlaceholderSignMessage;
import net.knarcraft.placeholdersigns.container.SignTextCopyRequest;
import net.knarcraft.placeholdersigns.util.TabCompleteHelper;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.block.sign.Side;
import org.bukkit.block.sign.SignSide;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

/**
 * A command for copying a sign's text
 */
public class CopySignTextCommand implements TabExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender commandSender, @NotNull Command command, @NotNull String s,
                             @NotNull String[] arguments) {
        PlaceholderSigns placeholderSigns = PlaceholderSigns.getInstance();
        StringFormatter stringFormatter = placeholderSigns.getStringFormatter();
        if (!(commandSender instanceof Player player)) {
            stringFormatter.displayErrorMessage(commandSender, PlaceholderSignMessage.ERROR_PLAYER_ONLY);
            return false;
        }

        Block targetBlock = player.getTargetBlockExact(7);
        if (targetBlock == null || !(targetBlock.getState() instanceof Sign sign)) {
            stringFormatter.displayErrorMessage(commandSender, PlaceholderSignMessage.ERROR_NOT_LOOKING_AT_SIGN);
            return false;
        }

        Side side = null;
        Integer sourceLine = null;
        Integer destinationLine = null;

        if (arguments.length > 0) {
            try {
                side = Side.valueOf(arguments[0]);
            } catch (IllegalArgumentException exception) {
                // Choose the side the player is currently looking at
                if (arguments[0].equalsIgnoreCase("this")) {
                    SignSide standingOn = sign.getTargetSide(player);
                    side = sign.getSide(Side.FRONT).equals(standingOn) ? Side.FRONT : Side.BACK;
                } else {
                    return false;
                }
            }
        }
        if (arguments.length > 1) {
            try {
                sourceLine = Integer.parseInt(arguments[1]) - 1;
            } catch (NumberFormatException exception) {
                return false;
            }
        }
        if (arguments.length > 2) {
            try {
                destinationLine = Integer.parseInt(arguments[2]) - 1;
            } catch (NumberFormatException exception) {
                return false;
            }
        }

        SignTextCopyRequest signCopyRequest = new SignTextCopyRequest(player, sign, side, sourceLine, destinationLine);
        placeholderSigns.getRequestHandler().addSignTextCopyRequest(signCopyRequest);

        stringFormatter.displaySuccessMessage(commandSender, PlaceholderSignMessage.SUCCESS_CLICK_SIGN_TO_PASTE);
        return true;
    }

    @Nullable
    @Override
    public List<String> onTabComplete(@NotNull CommandSender commandSender, @NotNull Command command, @NotNull String s,
                                      @NotNull String[] arguments) {
        if (arguments.length == 1) {
            List<String> signSides = TabCompleteHelper.getSignSides();
            signSides.add("this");
            return TabCompletionHelper.filterMatchingStartsWith(signSides, arguments[0]);
        } else if (arguments.length < 4) {
            return TabCompletionHelper.filterMatchingStartsWith(TabCompleteHelper.getLineNumbers(),
                    arguments[arguments.length - 1]);
        } else {
            return new ArrayList<>();
        }
    }

}
