package net.knarcraft.placeholdersigns.handler;

import net.knarcraft.placeholdersigns.container.SignCopyRequest;
import net.knarcraft.placeholdersigns.container.SignLineChangeRequest;
import net.knarcraft.placeholdersigns.container.SignTextCopyRequest;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.Map;

/**
 * A class for keeping track of placeholder sign requests
 */
public class PlaceholderSignRequestHandler {

    private final @NotNull Map<Player, SignLineChangeRequest> signChangeRequests;
    private final @NotNull Map<Player, SignTextCopyRequest> signTextCopyRequests;
    private final @NotNull Map<Player, SignCopyRequest> signCopyRequests;

    /**
     * Instantiates a new placeholder sign request handler
     */
    public PlaceholderSignRequestHandler() {
        this.signChangeRequests = new HashMap<>();
        this.signCopyRequests = new HashMap<>();
        this.signTextCopyRequests = new HashMap<>();
    }

    /**
     * Registers a sign copy request
     *
     * <p>A sign copy request is basically the result of running the copySign command, which must be stored until the
     * player clicks a sign.</p>
     *
     * @param request <p>The sign copy request to register</p>
     */
    public void addSignCopyRequest(@NotNull SignCopyRequest request) {
        this.signCopyRequests.put(request.player(), request);
    }

    /**
     * Gets a sign copy request
     *
     * @param player <p>The player to get the request for</p>
     * @return <p>The sign copy request, or null if not found</p>
     */
    @Nullable
    public SignCopyRequest getSignCopyRequest(@NotNull Player player) {
        return this.signCopyRequests.remove(player);
    }

    /**
     * Registers a sign change request
     *
     * <p>A sign change request is basically the result of running the editSign command, which must be stored until the
     * player clicks a sign.</p>
     *
     * @param request <p>The sign change request to register</p>
     */
    public void addSignChangeRequest(@NotNull SignLineChangeRequest request) {
        this.signChangeRequests.put(request.player(), request);
    }

    /**
     * Gets a sign change request
     *
     * @param player <p>The player to get the request for</p>
     * @return <p>The sign change request, or null if not found</p>
     */
    @Nullable
    public SignLineChangeRequest getSignChangeRequest(@NotNull Player player) {
        return this.signChangeRequests.remove(player);
    }

    /**
     * Registers a sign text copy request
     *
     * @param signCopyRequest <p>The request to register</p>
     */
    public void addSignTextCopyRequest(@NotNull SignTextCopyRequest signCopyRequest) {
        this.signTextCopyRequests.put(signCopyRequest.player(), signCopyRequest);
    }

    /**
     * Gets a sign text copy request
     *
     * @param player <p>The player to get the request for</p>
     * @return <p>The sign text copy request, or null if not found</p>
     */
    @Nullable
    public SignTextCopyRequest getSignTextCopyRequest(@NotNull Player player) {
        return this.signTextCopyRequests.remove(player);
    }

}
